use std::fs;

fn main() {
    let mut words = fs::read_to_string("./wordle.txt")
        .expect("Cannot read file.")
        .split('\n')
        .filter(|&word| word.trim().len() == 5)
        .map(|word| word.trim().to_lowercase())
        .collect::<Vec<String>>();

    let exact = ".u...";
    let close: Vec<(char, usize)> = vec![
        ('.', 0),
        ('m', 0),
        ('o', 1),
        ('h', 2),
        ('u', 3),
        ('r', 2),
        ('o', 4),
    ];
    let banned = "ptbenisa";

    words = words
        .iter()
        .filter(|&word| check_exact(word, exact))
        .filter(|&word| check_close(word, &close))
        .filter(|&word| check_banned(word, banned))
        .map(|word| word.to_owned())
        .collect::<Vec<String>>();

    print_guesses(words);
    // let mut count: usize = 0;
}

fn check_exact(word: &String, exact: &str) -> bool {
    assert!(
        word.len() == exact.len(),
        "Exact match string has invalid length!"
    );

    for (w, e) in word.chars().into_iter().zip(exact.chars()) {
        if e == '.' {
            continue;
        }

        if w != e {
            return false;
        }
    }
    return true;
}

fn check_close(word: &String, close: &Vec<(char, usize)>) -> bool {
    for (character, index) in close {
        // igore initialization value
        if *character == '.' {
            continue;
        }

        // character cant be in the same index
        if *character == word.chars().nth(*index).unwrap() {
            return false;
        }

        // character must appear in word to be valid guess
        if !word.contains(*character) {
            return false;
        }
    }
    return true;
}

fn check_banned(word: &String, banned: &str) -> bool {
    for b in banned.chars() {
        if word.contains(b) {
            return false;
        }
    }
    return true;
}

fn word_value(word: &String) -> usize {
    let mut value: usize = 0;

    // Give value to vowels in word
    let vowels = "aeiou";
    for vowel in vowels.chars() {
        if word.contains(vowel) {
            value = value + 2;
        }
    }

    if word.chars().last().unwrap() == 's' {
        value = value + 3;
    }

    return value;
}

fn print_guesses(words: Vec<String>) {
    if words.is_empty() {
        println!("Out of guesses!!!");
        return;
    }

    let values = words
        .iter()
        .map(|word| word_value(word))
        .collect::<Vec<usize>>();

    let &max = values.iter().max().unwrap();

    println!("Total Words: {}\nBest Guesses:", words.len());
    for (word, value) in words.iter().zip(values) {
        if value == max {
            println!("{}", word);
        }
    }
}
